#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include <stdio.h> 
#include <unistd.h> 
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>


#define LM_ERROR_MSG "ERRORE: impossibile leggere i sensori !\n\n"                                             \
                     " 1) i moduli i2c del kernel per il controllo deigli sensori LM devono essere caricati\n" \
		     " 2) il comando sensors deve essere installato e in path\n"                               \
	             " 3) lmsens.sh deve essere in path, controlla con il comando which lmsens.sh\n"           \
	             " 4) il file $HOME/.lmsens-input.tmp deve essere leggibile e scrivibile dal tuo utente"   

void
on_aggiorna_pressed(GtkButton       *button,
		    gpointer         user_data)
{
 FILE * file;
 GtkWidget * pb;
 char strn[80], strn2[80], * name, * text, * str[12], * s, tmp[120]; 
 const char * wg[7] = {"ptemp1", "ptemp2", "ptemp3", "pfan1", "pfan2", "pfan3", "apmbar"};
 const char * lb[9] = {"vtemp1", "vtemp2", "vtemp3", "vfan1", "vfan2", "vfan3", "maxT", "maxV", "vapm"};
 float valori[6], apm;
 static float max[6], min[6];
 static int mode = STD;
 int i, j, n; 
 long flun;

 /* Disattiva l'aggiornamento automatico
  * a vantaggio di quello manuale
  */
 //if(Freq == 0)
 //{
 // g_print("\nEscaped");
 // return;
 //}

 /* Inizializzo i massimi */

 max[3] = MAX_RPM;
 max[4] = MAX_RPM;
 max[5] = MAX_RPM;

 if(VisualT == STD)
 {
  for ( i = 0; i < 3; i++)
   max[i] = MAX_TEMP;
  mode = STD;
  pb = lookup_widget(Base, lb[6]);
  gtk_label_set(GTK_LABEL(pb), " ");
  pb = lookup_widget(Base, lb[7]);
  gtk_label_set(GTK_LABEL(pb), " ");
 }
 else if(mode == STD)
 {
  for ( i = 0; i < 3; i++)
  {
   max[i] = 0;
   min[i+3] = MAX_RPM;
  }
  mode = !STD;
  pb = lookup_widget(Base, lb[6]);
  gtk_label_set(GTK_LABEL(pb), "max");
  pb = lookup_widget(Base, lb[7]);
  gtk_label_set(GTK_LABEL(pb), "min");
 }

 /* Eseguo lo script e creo un file */
 if( my_system("lmsens.sh > $HOME/.lmsens-input.tmp") )
 {
  g_print("\n\n");
  show_error_msg(LM_ERROR_MSG);
  g_print("\n\n");
  
  /* Sospendo l'aggiornamento automatico */
  aggiorna_timer(0);
  return;
 }

 /* Costruisco il nome del file ricostruendolo
  * a partire dalla variabile $HOME
  */
 n = strlen( getenv("HOME") ) + strlen( "/.lmsens-input.tmp" );
 name = (char *)malloc(n + 80);
 sprintf( name, getenv("HOME") );
 strcat( name, "/.lmsens-input.tmp" );

 /* Apro, leggo e chiudo il file 
  * con dentro le statistiche
  */
 file = fopen(name, "rt");
 if(file == NULL)
 {
  sprintf(tmp, "\nERRORE: non riesco ad aprire %s", name);
  show_error_msg(tmp);
  return;
 }
 
 fgets(strn, 80, file);
 n = strlen(strn);
 fgets(strn2, 80, file);
 n += strlen(strn2);
 flun = getfilelenght(file);
 fseek(file, n, SEEK_SET);
 g_print("\nRead file %d of %d", n, (int)flun);


 /* 2000-03-30 Roberto A. Foglietta
  * Inserito su segnalazione di Marco Cardinale
  * BUG FIX: quando i sensori non sono leggibili
  *          evita il segmentation dell'applicazione
  */
 if(flun < n+16)
 {
  g_print("\n\n");
  show_error_msg(LM_ERROR_MSG);
  g_print("\n\n");
  
  /* Sospendo l'aggiornamento automatico */
  aggiorna_timer(0);
  return;
 }

 text = (char *)malloc( flun );
 if(text == NULL)
 {
  sprintf(tmp, "\nERRORE: non posso allocare lo spazio (%ld, %d)", flun, n);
  show_error_msg(tmp);
  free(text);
  return;
 }
 else
  g_print("\nAllocati %ld bytes, dopo %d", flun, n);
 flun -= n+1;
 fread(text, 1, (size_t)flun, file);
 text[flun] = '\0';
 fclose(file);


 /* Interpretazione dell'APM */
 g_print("\nAPM Status: %s", strn2);
 if ( strstr(strn2, "no system battery") )
 {
  pb = lookup_widget(Base, wg[6]);
  gtk_progress_configure (GTK_PROGRESS (pb), -1, -1, 1);
  gtk_progress_bar_update(GTK_PROGRESS_BAR (pb), 1);
  pb = lookup_widget(Base, lb[8]);
  gtk_label_set(GTK_LABEL(pb), " N/D   ");
 }
 else if ( strstr(strn2, "battery status") )
 {
  s = strn2;
  /* Trovo l'inizio della misura percentuale */
  i = 0; while(s[i] != ':' && i < 40) i++;
  s = &s[i+1];
  /* Trovo la fine della misura percentuale */
  i = 1; while(s[i] != '(' && i < 10) i++;
  s[i] = 0;
  /* Converto la misura percentuale */
  apm = atof(s)/100;
  g_print("APM Rilevamento: %s [%2.0f%%]", s, apm*100);

  pb = lookup_widget(Base, wg[6]);
  gtk_progress_configure (GTK_PROGRESS (pb), -1, -1, 1);
  gtk_progress_bar_update(GTK_PROGRESS_BAR (pb), apm);

  pb = lookup_widget(Base, lb[8]);
  if ( strstr(strn2, "on-line") )
   gtk_label_set(GTK_LABEL(pb), " ON-LINE  ");
  else
  {
   g_print("\n strn2: %s", strn2);
   s = &s[i+1];
   g_print("\n s: %s", s);
   i = 1; while(s[i] != ')' && i < 10) i++; 
   s[i] = 0;
   g_print("\n s: %s", s);
   sprintf(tmp, " %s  ", s);
   gtk_label_set(GTK_LABEL(pb), tmp);
  }
 }

 g_print("\nTimer frequency: %d sec", FREQ);


 /* Preparazione del testo */
 for(i = 0; i < flun-1; i++)
 {
  if( text[i] == '\n' )
   text[i] = ':';
  else if( text[i] == ' ' && text[i+1] == ' ' )
  { 
   for(j = i; j < flun; j++)
    text[j] = text[j+1];
   i--;
   flun--;
   text[flun] = '\0';
  }
 }

 /* Separazione dei valori di tensione */
 s = strtok(text, ":");
 s += (s[0] == ' ') ? 1 : 0;
 str[0] = (s == NULL) ? NULL : strdup(s);
 for (i = 1; i < 12; i++)
 {
  s = strtok(NULL, ":"); 
  s += (s[0] == ' ') ? 1 : 0;
  str[i] = (s == NULL) ? NULL : strdup(s);
 }

 /* Costruzione del nuovo testo */
 memset(text, 0, flun); 
 n = 0;
 for (i = 0; i < 12; i += 2)
  n = sprintf(text, "%s\n %8s: %10s", text, str[i], str[i+1]);
 
  
 g_print("\nLMsenStats: %s", strn);
 g_print("\nLMsensFile: %s", name);
 g_print("\n%s\n", text);
 free(name);

 /* Raccolgo i valori e li trasformo
  * in numeri interi
  */
 valori[0] = atof(strtok(strn, ","));
 for (i = 1; i < 6; i++)
  valori[i] = atof(strtok(NULL, ",")); 

 /* Controllo la validit dei valori 
  * e li stampo sulla console per debug
  */
 for (i = 0; i < 6; i++)
 {
  g_print("\ntemp%d: %5.0f", i+1, valori[i]);
  valori[i] = (valori[i] < 0) ? 0 : valori[i];
  if (valori[i] >= max[i]) 
  {
   max[i] = valori[i];
   valori[i] = (max[i]) ? 1 : 0;
  }
  else if(valori[i] <= min[i])
  {
   min[i] = valori[i];
   valori[i] /= max[i];
  }
  else 
   valori[i] /= max[i];
  g_print(",  %2.3f  [%4.0f - %4.0f]", valori[i], min[i], max[i]);

  pb = lookup_widget(Base, wg[i]);
  gtk_progress_configure (GTK_PROGRESS (pb), -1, -1, max[i]);
  gtk_progress_bar_update(GTK_PROGRESS_BAR (pb), valori[i]);
 }

 for (i = 0; i < 3; i++)   
 {
  if(VisualT != STD)
  {
   if(max[i] < 100)
    sprintf(tmp, " %3.1f C", max[i]);
   else
    sprintf(tmp, "%5.0f C ", max[i]);
  }
  else
   sprintf(tmp, " %4.0f C", max[i]);
  pb = lookup_widget(Base, lb[i]);
  gtk_label_set(GTK_LABEL(pb), tmp);
 }

 /* I valori massimi della ventola andrebbero 
  * arrotondati ai 1000giri superiori
  */
 for (i = 3; i < 6; i++)   
 {
  if(VisualT != STD)
   sprintf(tmp, "%5.0f rpm", min[i]);
  else
   sprintf(tmp, " %2.0fK rpm", max[i]/1000);
  pb = lookup_widget(Base, lb[i]);
  gtk_label_set(GTK_LABEL(pb), tmp);
 }


 /* Stampo nella casella di testo */
 pb = lookup_widget(Base, "testo");
 gtk_editable_delete_text (GTK_EDITABLE (pb), 0, -1); 
 gtk_text_insert (GTK_TEXT (pb), gdk_fontset_load("*fixed*") , NULL, NULL, strdup(text), -1);
 free(text);
}

/************ MIE FUNZIONI ********************/

int my_system (const char *command) {
 int pid, status;
 
 if (command == 0)
     return 1;
 pid = fork();
 if (pid == -1)
     return -1;
 if (pid == 0) {
     char *argv[4];
     argv[0] = "sh";
     argv[1] = "-c";
     argv[2] = (char *)command;
     argv[3] = 0;
     execv("/bin/sh", argv);
     exit(127);
 }
 do {
     if (waitpid(pid, &status, 0) == -1) {
         if (errno != EINTR)
             return -1;
     } else
         return status;
 } while(1);
}

unsigned long getfilelenght(FILE * fp)
{
 long lun;
 
 fseek( fp, 0, SEEK_END);
 lun = ftell(fp);
 rewind(fp);
 
 return lun;
}


void aggiorna_timer(gint freq)
{
 int i, idx;
 GtkWidget * wchk;
 static char * chk[9]  = {"manuale", "_10sec", "_15sec", 
                          "_20sec",  "_30sec", "_1min", 
			  "_2min",   "_5min",  "_10min"};
 static int time[9] = {0, 10, 15, 20, 30,
                       60, 120, 300, 600 }, inib = 0;

 if(inib)
  return; 

 if(Freq != freq)
 {
  Freq = freq;
  if(Timer != 0)
   gtk_timeout_remove(Timer);
  Timer = 0;

  /* Determino approsimativamente quale
   * aggiornamento selezionare nel menu
   */
  idx = 0;
  for(i = 1; i < 9 && !idx; i++)
   if(Freq > time[i-1] && Freq <= time[i])
    idx = i;  
  if(Freq > time[8])
   idx = 8;
  g_print("\n\nFrequenza selezionata: %d sec", time[idx]);

  /* Aggiorno il menu delle frequenze 
   * il quale per emmette un segnale 
   * che mi devo occupare di ignorare
   */
  inib = 1;
  wchk = lookup_widget(Base, chk[idx]);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (wchk), TRUE);
  inib = 0;

  if(Freq != 0)
  {
   Timer = gtk_timeout_add( (guint32)Freq*1000, (GtkFunction)on_aggiorna_pressed, NULL );
   g_print("\nTimer frequency: %d sec", FREQ);
  }
  else
   g_print("\nTimer frequency: SOSPESA");
 }
}


void show_info_msg (void)
{
 GtkWidget * messaggio;
 char * infomsg;

 infomsg = (char *)malloc(strlen(RVERSN)+strlen(AUTHOR)+strlen(EMAIL)+strlen(WEBPAGE)+16);
 if( infomsg )
 {
  sprintf(infomsg, "\n%s\n\n%s\n%s\n%s\n", RVERSN, AUTHOR, EMAIL, WEBPAGE);
  Info = create_informazioni();
  messaggio = lookup_widget(Info, "infolabel");
  gtk_label_set (GTK_LABEL(messaggio), infomsg);
 }
 else
  Info = create_informazioni();

 MOSTRA_ON_MOUSE(Info);
 return;
}

void show_error_msg (const char * errmsg)
{
 GtkWidget * messaggio, * msg_errore;

 /* Inutile creare messaggi di errore
  * all'infinito si rischia solo un 
  * blocco della macchina.
  */
 if(Nerr >= MAX_ERR)
  return;

 g_print("\n\n%s\n", errmsg);

 msg_errore = create_errore();
 messaggio = lookup_widget(msg_errore, "error_msg");
 gtk_label_set (GTK_LABEL(messaggio), errmsg);
 MOSTRA_ON_MOUSE(msg_errore);
}

/************ FINE MIE FUNZIONI ********************/

void
on_gsensor_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
 g_print("\n");
 aggiorna_timer(0);
 gtk_exit(0);
}


void
on_Vstandard_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 if(VisualT != STD)
 {
  VisualT = STD;
  on_aggiorna_pressed(NULL, NULL);
 }
}


void
on_Vmaxmin_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 if(VisualT == STD)
 {
  VisualT = !STD;
  on_aggiorna_pressed(NULL, NULL);
 }
}


void
on_manuale_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 aggiorna_timer(0);
}


void
on_10sec_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data) 
{
 aggiorna_timer(10);
}


void
on_15sec_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 aggiorna_timer(15);
}


void
on_20sec_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 aggiorna_timer(20);
}


void
on_30sec_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 aggiorna_timer(30);
}


void
on_1min_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 aggiorna_timer(60);
 /* A questo punto i massimi 
  * non hanno pi alcuna importanza
  */
 // VisualT = STD;
}


void
on_2min_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 aggiorna_timer(120);
}


void
on_5min_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 aggiorna_timer(300);
}


void
on_10min_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 aggiorna_timer(600);
}


void
on_info_released                       (GtkButton       *button,
                                        gpointer         user_data)
{
 gtk_widget_destroy (GTK_WIDGET (Info));
}


void
on_impostazioni_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

void
on_autoreprg_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
 show_info_msg ();
}

void
on_ok_errormsg_pressed (GtkButton * button, gpointer user_data)
{
 GtkWidget * msg_errore;
  
 msg_errore = lookup_widget (GTK_WIDGET (button), "errore");
 gtk_widget_destroy (msg_errore);
}

void
on_errore_destroy                      (GtkObject       *object,
                                        gpointer         user_data)
{
 Nerr--;
}


void
on_errore_show                         (GtkWidget       *widget,
                                        gpointer         user_data)
{
 Nerr++;
}



